"""This tracks the last commit and prints out the results."""
import pytest

# import json
# import requests
# import re
# import subprocess
#
# from git import Repo

global USERTOKEN
USERTOKEN = " "

pytest_plugins = "pytester"


@pytest.fixture
def TOKEN():
    return USERTOKEN


# def pytest_configure(config):
#     global SLUG
#     rawProcess = subprocess.run(
#         ["git", "config", "--get", "remote.origin.url"], stdout=subprocess.PIPE
#     )
#     output = rawProcess.stdout.decode("utf-8")
#     regexMatches = re.search(r".*(/|:)(.+?/.+?)\.git", output)
#     SLUG = regexMatches.group(2)
#
#
# def pytest_addoption(parser):
#     """Print stuff to header with --track"""
#     group = parser.getgroup("track")
#     group.addoption(
#         "--track",
#         action="store_true",
#         help="pytest-blame help \n--track: show last git commit",
#     )
#
#
# def getstatus(sha):
#     """Get status of CI check from github"""
#     # request data of the specific sha
#     response = requests.get(
#         "https://api.github.com/repos/" + SLUG + "/statuses/" + str(sha)
#     )
#     # read json data and convert it to list
#     statuses = json.loads(response.text)
#     # statuses will be an empty list if the state is failling or pending
#     if statuses == []:
#         check = "failure"
#     else:
#         state = statuses[0]
#         check = state["state"]
#     return check
#
#
# # pylint: disable=E1101
# def pytest_report_header():
#     """Display github commits"""
#     if pytest.config.getoption("track"):
#         PATH = "."
#         repo = Repo(PATH)
#         commits = list(repo.iter_commits())
#         for i in range(len(commits)):
#             # check if the most recent commit is passing
#             if getstatus(commits[i].hexsha) == "success" and i == 0:
#                 print(
#                     "\nThe most recent commit is passing: ",
#                     "https://github.com/" + SLUG + "/commit/" + commits[i].hexsha,
#                     "\n",
#                     commits[i].author,
#                     ":",
#                     commits[i].message,
#                 )
#                 break
#             # check if no passing commit
#             elif i == len(commits) - 1 and getstatus(commits[i].hexsha) == "failure":
#                 print(
#                     "\nCan't find passing commit, the most recent commit is failing: ",
#                     "https://github.com/" + SLUG + "/commit/" + commits[0].hexsha,
#                     "\n",
#                     commits[0].author,
#                     ":",
#                     commits[0].message,
#                 )
#                 break
#             # check if current commit is failing
#             elif getstatus(commits[i].hexsha) == "failure":
#                 pass
#             # find the most recent passing commit
#             else:
#                 passingcommits = (
#                     "\nMost recent passing commit: "
#                     + "https://github.com/"
#                     + SLUG
#                     + "/commit/"
#                     + commits[i].hexsha
#                     + "\n"
#                     + str(commits[i].author)
#                     + ": "
#                     + str(commits[i].message)
#                     + "\n"
#                     + "--------------------------------"
#                 )
#                 faillingcommits = ""
#                 # looping through all failing commits
#                 while i > 0:
#                     faillingcommits = (
#                         "\nFailing commit: "
#                         + "https://github.com/"
#                         + SLUG
#                         + "/commit/"
#                         + commits[i].hexsha
#                         + "\n"
#                         + str(commits[i - 1].author)
#                         + ": "
#                         + str(commits[i - 1].message)
#                         + faillingcommits
#                     )
#                     i -= 1
#                 print(
#                     passingcommits,
#                     faillingcommits,
#                     "\nThe last one is the most recent commit\n",
#                 )
#                 break
#     # give msg a default value
#     else:
#         print("\nCan't find commits")
#
#
# @pytest.fixture
# def no_arguments():
#     """Return no command-line arguments"""
#     return []



""" Tests the pytest_blame.py plugin for pytest """

import subprocess
import os
import pytest

# pylint: disable=W0621
@pytest.fixture()
def sample_test(testdir):
    """temporal test file"""
    testdir.makepyfile(
        """
        def test_pass():
            assert 1 == 1
    """
    )
    return testdir


def test_addoption(sample_test):
    """test addoption"""
    os.chdir(f"{os.environ['HOME']}/.local")
    subprocess.run(["git", "clone", "git@github.com:inTestiGator/test-repository.git"])
    os.chdir(f"{os.environ['HOME']}/.local/test-repository/")
    result = sample_test.runpytest("--track")
    print(result.stdout)
    result.stdout.fnmatch_lines(["*"])
    print(result.ret)
    assert result.ret == 5



"""This tracks the last commit and prints out the results."""
import re
import subprocess
import os
import pytest
import requests
from git import Repo


# pylint: disable=W0601
def pytest_configure(config):
    """Get the slug of the user's repository and the user's github token and save
    them both in global variables."""
    global USERTOKEN
    global SLUG
    if config.pluginmanager.hasplugin("blame"):
        rawProcess = subprocess.run(
            ["git", "config", "--get", "remote.origin.url"], stdout=subprocess.PIPE
        )
        if rawProcess == "":
            raise Exception(
                "No git repository found. Please run pytest-blame \
                            from inside a git repo."
            )
        output = rawProcess.stdout.decode("utf-8")
        regexMatches = re.search(r".*(/|:)(.+?/.+?)\.git", output)
        SLUG = regexMatches.group(2)
        USERTOKEN = os.environ["GITHUB_OAUTH_TOKEN"]


def pytest_addoption(parser):
    """Create --track option to run the plugin"""
    group = parser.getgroup("track")
    group.addoption(
        "--track",
        action="store_true",
        help="pytest-blame help \n--track: show last git commit",
    )


def getstatus(sha, TOKEN):
    """Get status of CI check from github"""
    # request data of the specific sha
    response = requests.get(
        "https://api.github.com/repos/" + SLUG + "/commits/" + str(sha) + "/check-runs",
        headers={
            "Authorization": f"token {TOKEN}",
            "Accept": "application/vnd.github.antiope-preview+json",
        },
    )
    raw = response.json()
    if raw["total_count"] != 0:
        status = raw["check_runs"][0]["conclusion"]
        return status
    return "failure"


# pylint: disable=E1101, C0200
def pytest_report_header():
    """Adjust the header of the report to display the information for last passing
    commit, failing commits, etc."""
    if pytest.config.getoption("track"):
        PATH = "."
        repo = Repo(PATH)
        commits = list(repo.iter_commits())
        for i in range(len(commits)):
            # check if the most recent commit is passing
            if getstatus(commits[i].hexsha, USERTOKEN) == "success" and i == 0:
                print(
                    "\nThe most recent commit is passing: ",
                    "https://github.com/" + SLUG + "/commit/" + commits[i].hexsha,
                    "\n",
                    commits[i].author,
                    ":",
                    commits[i].message,
                )
                break
            # check if no passing commit
            elif (
                # pylint: disable=C0330
                i == len(commits) - 1
                and getstatus(commits[i].hexsha, USERTOKEN) == "failure"
            ):
                print(
                    "\nCan't find passing commit, the most recent commit is failing: ",
                    "https://github.com/" + SLUG + "/commit/" + commits[0].hexsha,
                    "\n",
                    commits[0].author,
                    ":",
                    commits[0].message,
                )
                break
            # check if current commit is failing
            elif getstatus(commits[i].hexsha, USERTOKEN) == "failure":
                pass
            # traceback to find the last passing commit
            else:
                passingcommits = (
                    "\nMost recent passing commit: "
                    + "https://github.com/"
                    + SLUG
                    + "/commit/"
                    + commits[i].hexsha
                    + "\n"
                    + str(commits[i].author)
                    + ": "
                    + str(commits[i].message)
                    + "\n"
                    + "--------------------------------"
                )
                faillingcommits = ""
                # looping through all failing commits
                while i > 0:
                    faillingcommits = (
                        "\nFailing commit: "
                        + "https://github.com/"
                        + SLUG
                        + "/commit/"
                        + commits[i].hexsha
                        + "\n"
                        + str(commits[i - 1].author)
                        + ": "
                        + str(commits[i - 1].message)
                        + faillingcommits
                    )
                    i -= 1
                print(
                    passingcommits,
                    faillingcommits,
                    "\nThe first one is the most recent commit\n",
                )
                break
    # give msg a default value
    else:
        print("\nCan't find commits")



"""This file is used to deploy the plugin"""
import io
import os
from setuptools import setup

install_requires = ["pygithub", "pytest>=4.4.0", "gitpython", "requests"]


def read(filename):
    """This function takes in a filepath and reads the file"""
    filepath = os.path.join(os.path.dirname(__file__), filename)
    with io.open(filepath, mode="r", encoding="utf-8") as f:
        return f.read()


setup(
    name="pytest-blame",
    version="0.1.7",
    description="A pytest plugin helps developers to debug by providing useful commits history.",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Lancaster Wu, Spencer Huang, Carson Quigley, Patrick Palad, Paul Livingston",
    author_email="wuj@allegheny.edu, huangs@allegheny.edu, quigleyc@allegheny.edu, "
    + "paladp@allegheny.edu, livingstonp@allegheny.edu",
    url="https://github.com/inTestiGator/pytest-blame",
    license="GNU",
    platforms="any",
    install_requires=install_requires,
    py_modules=["pytest_blame"],
    entry_points={"pytest11": ["blame = pytest_blame"]},
)
